@@ -4,31 +4,48 @@ use Carp;
 use Mouse;
 use Config::Model::Exception;
 use UNIVERSAL;
-use Path::Tiny;
+use Path::Tiny 0.054;
 use File::Path;
 use Log::Log4perl qw(get_logger :levels);
-use 5.10.1;
+use 5.20.1;
+
+use feature qw/postderef signatures/;
+no warnings qw/experimental::postderef experimental::signatures/;
 
 extends 'Config::Model::Backend::PlainFile';
 
 my $logger = get_logger("Backend::Dpkg::Root");
 
-sub read_hash {
-    my ( $self, $obj, $elt, $file, $check, $args ) = @_;
+my %dispatch = (
+    patches => \&read_patch_series,
+    install => \&read_install_files,
+);
 
-    if ( $elt eq 'patches' ) {
-        my $patch_dir = $args->{root} . $args->{config_dir} . "patches";
-        $logger->info("Checking patches directory ($patch_dir)");
+around read_hash => sub ( $orig, $self, $obj, $elt, $file, $check, $args ) {
 
-        $self->read_patch_series( $obj, $check, $patch_dir, $args );
-    }
-    else {
-        $self->SUPER::read_hash(@_);
+    $logger->info("called for $elt ".$obj->location." file $file" );
+    my $method = $dispatch{$elt} // $orig;
+
+    $self->$method( $obj, $elt, $file, $check, $args );
+};
+
+sub read_install_files ( $self, $hash, $elt, $file, $check, $args ) {
+    my $dir = path($args->{root} . $args->{config_dir});
+
+    return unless $dir->exists;
+
+    $logger->info("Checking $elt directory ($dir) for ".$hash->location );
+    foreach my $file ($dir->children(qr/\.install$/)) {
+        my $pkg =  $file->basename(qr/\.install$/);
+        $logger->info("install: found $pkg install file");
+        # Just create the element. The read backend will kick in by itself
+        $hash->fetch_with_id($pkg);
     }
 }
 
-sub read_patch_series {
-    my ( $self, $hash, $check, $patch_dir, $args ) = @_;
+sub read_patch_series ( $self, $hash, $elt, $file, $check, $args ) {
+    my $patch_dir = $args->{root} . $args->{config_dir} . "patches";
+    $logger->info("Checking patches directory ($patch_dir)");
 
     my $series_files = "$patch_dir/series";
 
