@@ -43,20 +43,16 @@ sub read {
     my $patch_dir = $args{root} . $args{config_dir};
     my $check     = $args{check};
     my $node      = $args{object};
-    my $file_path = $args{file_path};
 
-    my $patch_name = $node->index_value || $args{config_file};
+    my $backend_arg = $self->instance->backend_arg;
+    my $patch_name = $node->index_value || $backend_arg;
 
-    my $patch_file
-        = ($file_path and -e $file_path) ? $file_path
-        : -e "$patch_dir$file_path"      ? "$patch_dir$file_path"
-        :                                  "$patch_dir$patch_name" ;
+    my $patch_file = "$patch_dir$patch_name" ;
     $self->{patch_file} = $patch_file;
 
     $logger->info("Parsing patch $patch_file");
     my $patch_io = IO::File->new($patch_file)
-      || Config::Model::Exception::Syntax->throw(
-        message => "cannot read patch $patch_file" );
+      || Config::Model::Exception::User->throw(message => "cannot read patch $patch_file: $!" );
     $patch_io->binmode(':utf8');
 
     my ( $header, $diff ) = ( [],[] );
@@ -85,7 +81,7 @@ sub read {
     } ;
 
     if (@$header) {
-        $c = eval { $self->parse_dpkg_lines( $file_path, $header, $check, 0, $store_stuff ); };
+        $c = eval { $self->parse_dpkg_lines( $patch_file, $header, $check, 0, $store_stuff ); };
         my $e = $@;
         if ( ref($e) and $e->isa('Config::Model::Exception::Syntax') ) {
             $e->parsed_file( $patch_file );
