@@ -37,6 +37,7 @@ my $madison_endpoint = "https://$madison_host/madison";
 # See https://www.debian.org/doc/packaging-manuals/virtual-package-names-list.txt
 # updated from 30 Jul 2014 version
 my @virtual_list = qw/
+ adventure
  audio-mixer
  awk
  boom-engine
@@ -62,6 +63,7 @@ my @virtual_list = qw/
  httpd
  httpd-cgi
  httpd-wsgi
+ httpd-wsgi3
  ident-server
  imap-client
  imap-server
@@ -118,6 +120,11 @@ my @virtual_list = qw/
  telnet-server
  time-daemon
  ups-monitor
+ virtual-mysql-client
+ virtual-mysql-client-core
+ virtual-mysql-server
+ virtual-mysql-server-core
+ virtual-mysql-testsuite
  wish
  wordlist
  www-browser
@@ -141,9 +148,6 @@ push @virtual_list,
 
 my %virtual_hash = map {( $_ => 1); } @virtual_list;
 
-use vars qw/$test_filter/ ;
-$test_filter = ''; # reserved for tests
-
 my %debian_map;
 my $version = \%Module::CoreList::version;
 
@@ -248,7 +252,7 @@ arch_restriction: '[' osarch(s) ']'
 
 dep_version: '(' oper version ')' { $return = [ $item{oper}, $item{version} ] ;}
 
-pkg_name: /[a-z0-9][a-z0-9\+\-\.]+(?=\s|\Z|\(|\[)/
+pkg_name: /[a-z0-9][a-z0-9\+\-\.]+(?=[\s([|]|\Z)/
     | /\S+/ { $add_error->("bad package name", $item[1]) ;}
 
 oper: '<<' | '<=' | '=' | '>=' | '>>'
@@ -336,16 +340,17 @@ sub check_dependency {
     }
 
     my $old = $value ;
+    my @msgs;
 
     foreach my $dep (@dep_chain) {
         next unless ref($dep) ; # no need to check variables
-        $self->check_or_fix_pkg_name($apply_fix, $dep, $old) ;
-		$self->check_or_fix_essential_package($apply_fix, $dep, $old) ;
-		$self->check_or_fix_dep($apply_fix, $dep, $old) ;
+        $self->check_or_fix_pkg_name($apply_fix, $dep, $old, \@msgs) ;
+		$self->check_or_fix_essential_package($apply_fix, $dep, \@msgs) ;
+		$self->check_or_fix_dep($apply_fix, $dep, $old, \@msgs) ;
     }
 
 
-	$self->check_depend_chain($apply_fix, \@dep_chain, $old ) ;
+	$self->check_depend_chain($apply_fix, \@dep_chain, $old, \@msgs) ;
 
     # "ideal" dependency is always computed, but it does not always change
     my $new = $self->struct_to_dep(@dep_chain);
@@ -358,13 +363,14 @@ sub check_dependency {
 
     {
         no warnings 'uninitialized';
-        $self->_store_fix( $old, $new ) if $apply_fix and $new ne $old;
+        my $msg = join('; ', @msgs);
+        $self->_store_fix( $old, $new, $msg ) if $apply_fix and @msgs and $new ne $old;
     }
     return ($ok, $new) ;
 }
 
 sub check_debhelper_version {
-    my ($self, $apply_fix, $dep_info) = @_ ;
+    my ($self, $apply_fix, $dep_info, $msgs) = @_ ;
     my $dep_name = $dep_info->{name};
     my ($oper, $dep_v) = @{ $dep_info->{dep} || []};
 
@@ -373,6 +379,7 @@ sub check_debhelper_version {
     $logger->debug("checking '$dep_string' with lintian");
 
     # try to create compat_obj, but do not try twice (hence the exists test)
+    # compat_obj will be undef with 'cme edit dpkg-control'
     if (not exists $self->{_compat_obj} ) {
         # using mode loose because debian-control model can be used alone
         # and compat is outside of debian-control
@@ -385,44 +392,34 @@ sub check_debhelper_version {
     my $compat_value = $self->{_compat_obj}->fetch;
 
     my $min_dep = Lintian::Relation->new("debhelper ( >= $compat_value)") ;
-    $logger->debug("checking if ".$lintian_dep->unparse." implies ". $min_dep->unparse);
 
-    return if $lintian_dep->implies ($min_dep) ;
+    if ($lintian_dep->implies ($min_dep)) {
+        $logger->debug($dep_string." -> ".$lintian_dep->unparse." implies ". $min_dep->unparse);
+        return;
+    }
 
     $logger->debug("'$dep_string' does not imply debhelper >= $compat_value");
 
     # $show_rel avoids undef warnings
     my $show_rel = join(' ', map { $_ || ''} ($oper, $dep_v));
+    my $msg = "should be (>= $compat_value) not ($show_rel) because compat is $compat_value" ;
     if ($apply_fix) {
         $dep_info->{dep} = [ '>=', $compat_value] ; # notify_change called in check_value
         $logger->info("fixed debhelper dependency from "
             ."$dep_name $show_rel -> ".$min_dep->unparse." (for compat $compat_value)");
+        push $msgs->@*, $msg;
     }
     else {
         $self->{nb_of_fixes}++ ;
-        my $msg = "should be (>= $compat_value) not ($show_rel) because compat is $compat_value" ;
         $self->add_warning( $msg );
         $logger->info("will warn: $msg (fix++)");
     }
 }
 
-# used with dependency filter: an optional config parameter which enable a user
-# to clean dependency older than a specified release. Be default, dependency version
-# are not cleaned up before they are older than old_stable.
-my @deb_releases = qw/etch lenny squeeze wheezy jessie stretch buster/;
-
-my %deb_release_h ;
-while (@deb_releases) {
-    my $k = pop @deb_releases ;
-    my $regexp = join('|',@deb_releases,$k);
-    $deb_release_h{$k} = qr/$regexp/;
-}
-
 sub struct_to_dep {
     my $self = shift ;
     my @input = @_ ;
 
-    my $skip = 0 ;
     my @alternatives ;
     foreach my $d (@input) {
         my $line = '';
@@ -433,8 +430,6 @@ sub struct_to_dep {
             if ( $name) {
                 $line .= $name;
 
-                # skip test for relations like << or <
-                $skip ++ if defined $dep->[0] and $dep->[0] =~ /</ ;
                 $line .= " (@$dep)" if defined $dep->[1];
 
                 $line .= " [@$arch]" if $arch;
@@ -454,7 +449,7 @@ sub struct_to_dep {
 
     my $actual_dep = @alternatives ? join (' | ',@alternatives) : undef ;
 
-    return wantarray ? ($actual_dep, $skip) : $actual_dep ;
+    return $actual_dep ;
 }
 
 # @input contains the alternates dependencies (without '|') of one dependency values
@@ -462,19 +457,14 @@ sub struct_to_dep {
 
 # will modify @input (array of ref) when applying fix
 sub check_depend_chain {
-    my ($self, $apply_fix, $input, $old) = @_ ;
+    my ($self, $apply_fix, $input, $old, $msgs) = @_ ;
 
-    my ($actual_dep, $skip) = $self->struct_to_dep (@$input);
+    my $actual_dep = $self->struct_to_dep (@$input);
     my $ret = 1 ;
 
     return 1 unless defined $actual_dep; # may have been cleaned during fix
     $logger->debug("called with $actual_dep with apply_fix $apply_fix");
 
-    if ($skip) {
-        $logger->debug("skipping '$actual_dep': has a < relation ship") ;
-        return $ret ;
-    }
-
     foreach my $depend (@$input) {
         if (ref ($depend)) {
             # is a dependency (not a variable a la ${perl-Depends})
@@ -483,7 +473,7 @@ sub check_depend_chain {
             $logger->debug("scanning dependency $dep_name"
                 .(defined $dep_v ? " $dep_v" : ''));
             if ($dep_name =~ /lib[\w+\-]+-perl/) {
-                $ret &&= $self->check_perl_lib_dep ($apply_fix, $actual_dep, $depend,$input);
+                $ret &&= $self->check_perl_lib_dep ($apply_fix, $actual_dep, $depend, $input, $msgs);
                 last;
             }
         }
@@ -508,20 +498,20 @@ sub extract_cpan_version {
 # called through check_depend_chain
 # does modify $input when applying fix
 sub check_perl_lib_dep {
-    my ($self, $apply_fix, $actual_dep, $depend, $input) = @_;
+    my ($self, $apply_fix, $actual_dep, $depend, $input, $msgs) = @_;
     my $dep_name = $depend->{name};
     my ($oper, $dep_v) = @{ $depend->{dep} || []};
 
     $logger->debug("called for $dep_name with $actual_dep with apply_fix $apply_fix");
 
     my ($old_perl_dep) = grep { $_->{name} eq 'perl' } @$input;
-    my $old_perl_versioned_dep = $old_perl_dep->{dep}[1];
+    my $old_perl_versioned_dep = $old_perl_dep->{dep}[1] // '';
 
     # The dependency should be in the form perl (>= 5.10.1) | libtest-simple-perl (>= 0.88)".
     # cf http://pkg-perl.alioth.debian.org/policy.html#debian_control_handling
     # If the Perl version is not available in sid, the order of the dependency should be reversed
     # libcpan-meta-perl | perl (>= 5.13.10)
-    # because buildd will use the first available alternative
+    # because Debian build daemons use the first available alternative
 
     # check for dual life module, module name follows debian convention...
     my $cpan_name = $debian_map{$dep_name};
@@ -535,11 +525,10 @@ sub check_perl_lib_dep {
     my $removed    = Module::CoreList->removed_from($cpan_name) ;
     $logger->debug("dual life $dep_name is removed from perl $removed") if $removed;
 
-    return 1 if (defined $dep_v && $dep_v =~ m/^\$/) ; # version like ${foobar}
+    return 1 if (defined $dep_v && $dep_v =~ m/^\$/) ; # version like ${foobar} in lib dep
 
-    my %ideal_perl_dep = qw/name perl/ ;
+    my %ideal_perl_dep ;
     my %ideal_lib_dep ;
-    my @ideal_dep_chain = (\%ideal_perl_dep);
 
     my @res = $self->get_available_version( $dep_name);
 
@@ -575,6 +564,7 @@ sub check_perl_lib_dep {
 		version->parse( $cpan_dep_v )
 	);
 
+    # return when module is not and was never part of core
 	return 1 unless defined $v_decimal;
 
 	my $v_normal = version->new($v_decimal)->normal;
@@ -584,22 +574,60 @@ sub check_perl_lib_dep {
 		$logger->debug("dual life $dep_str found in Perl core $v_normal (req perl is $old_perl_versioned_dep)");
 	}
 
-    if ( not defined $dep_v and $old_perl_versioned_dep ) {
+    if ( not defined $dep_v and $old_perl_versioned_dep and $old_perl_versioned_dep gt $v_decimal) {
         # when alternate lib version is not defined (because the requirement is satisfied by all
         # available versions of the lib), the actual requirement is held by the versioned dep of perl
         # hence, it must be preserved
-		$logger->debug("preserving old perl versioned dep $old_perl_versioned_dep instead of $v_normal"
-                       ." for $dep_name");
+        my $p_data = Module::CoreList->find_version($v_decimal);
+        $dep_v = $p_data->{$cpan_name};
+        $depend->{dep} = [ '>=', $dep_v ];
+        $logger->debug("preserving old perl versioned dep $old_perl_versioned_dep instead of $v_normal"
+                           ." for $dep_name (implies dep > $dep_v)");
         $v_normal = $old_perl_versioned_dep;
     }
 
-	my ($has_older_perl) = $self->check_versioned_dep( { name => 'perl', dep => ['>=', $v_normal]} );
-	$ideal_perl_dep{dep} = [ '>=', $v_normal ] if $has_older_perl;
 
-    if ($removed or $deprecated or $has_older_perl) {
-        my ($has_older_lib) = $self->check_versioned_dep(  $depend );
+    my ($has_older_perl) = $self->check_versioned_dep( { name => 'perl', dep => ['>=', $v_normal]} );
+    my ($has_older_lib)  = $self->check_versioned_dep( $depend );
+
+    # Perl module is being or was removed from core
+    if ($deprecated or $removed) {
+        if ( $has_older_perl ) {
+            my $max_ver_normal = version->new($deprecated)->normal ;
+            $max_ver_normal =~ s/^v//;
+            $logger->debug("Found older perl for a soon to be removed lib. Using perl << $max_ver_normal dependency'");
+            $ideal_perl_dep{name} = 'perl';
+            $ideal_perl_dep{dep} = [ '<<', $max_ver_normal ];
+        }
+        else {
+            # e.g. perl (<< 5.8.0) i.e. lower than obsolete perl -> remove dependency from ideal perl dep
+            $logger->debug("Did not find older perl for an obsolete module. Perl dep should be removed");
+        }
+
+        # always specify the library dependency
         $ideal_lib_dep{name} = $dep_name;
         $ideal_lib_dep{dep} = [ '>=', $dep_v ] if $dep_v and $has_older_lib;
+
+    }
+    else {
+        # Perl module is part of core (this was tested before)
+        $ideal_perl_dep{name} = 'perl';
+        if ( $has_older_perl ) {
+            $logger->debug("Found older perl for a greater-than perl dep. Ideal perl dep is 'perl >= $v_normal'");
+            $ideal_perl_dep{dep} = [ '>=', $v_normal ];
+        }
+        else {
+            $logger->debug("Did no find older perl for a greater-than perl dep. Ideal perl dep is plain 'perl'");
+        }
+
+        # we need to specify the libray dependency even this one has no
+        # version requirement and the dependency is satisfied by Perl core
+        # because dual dependency will eventually be removed. Better keep this info
+        $ideal_lib_dep{name} = $dep_name;
+
+        if ($dep_v) {
+            $ideal_lib_dep{dep} = [ '>=', $dep_v ] if $dep_v and $has_older_lib;
+        }
     }
 
     my %perl_version =  $self->get_available_version( 'perl');
@@ -611,10 +639,13 @@ sub check_perl_lib_dep {
 		"older than perl in sid ($sid_perl_version)"
 	);
 
-	my @ordered_ideal_dep
-        = $removed || $deprecated  ? ( \%ideal_lib_dep )
-        : $has_older_perl_in_sid ? ( \%ideal_perl_dep, \%ideal_lib_dep )
-        :                          ( \%ideal_lib_dep, \%ideal_perl_dep ) ;
+    my $perl_dep_first = $has_older_perl_in_sid && ! $deprecated;
+
+	my @pre_ordered_ideal_dep
+        = $perl_dep_first   ? ( \%ideal_perl_dep, \%ideal_lib_dep  )
+        :                     ( \%ideal_lib_dep,  \%ideal_perl_dep ) ;
+
+	my @ordered_ideal_dep = grep {defined $_->{name} } @pre_ordered_ideal_dep;
 
 	my $ideal_dep = $self->struct_to_dep( @ordered_ideal_dep );
 
@@ -622,20 +653,22 @@ sub check_perl_lib_dep {
         unless defined $ideal_dep;
 
 	if ( $actual_dep ne $ideal_dep ) {
+        my $msg = "Dependency of dual life package should be '$ideal_dep' not '$actual_dep'";
 		if ($apply_fix) {
-			@$input = @ordered_ideal_dep ; # notify_change called in check_value
+			@$input =  @ordered_ideal_dep ; # notify_change called in check_value
             if ($logger->is_info) {
-                $logger->info("fixed dependency with: $ideal_dep, was ". $self->struct_to_dep($depend));
+                $logger->info("fixed dependency with: '$ideal_dep', was '$actual_dep'");
             }
+            push $msgs->@*, $msg;
 		}
 		else {
 			$self->{nb_of_fixes}++;
 			my $msg = "Dependency of dual life package should be '$ideal_dep' not '$actual_dep'";
+            if ($deprecated) {
+                $msg .= " (deprecated from perl $deprecated)" ;
+            }
             if ($removed) {
                 $msg .= " (removed from perl $removed)" ;
-            }
-            elsif ($deprecated) {
-                $msg .= " (deprecated from perl $deprecated)" ;
             }
 			$self->add_warning ($msg);
 			$logger->info("will warn: $msg (fix++)");
@@ -660,16 +693,11 @@ sub check_versioned_dep {
 
 	if ( @dist_version  # no older for unknow packages
 		 and defined $oper
-		 and $oper =~ />/
 		 and $vers !~ /^\$/  # a dpkg variable
 	 ) {
 		my $src_pkg_name = $self->grab_value("!Dpkg::Control source Source") ;
 
-		my $filter = $test_filter || $self->grab_value(
-			step => qq{!Dpkg my_config package-dependency-filter:"$src_pkg_name"},
-			mode => 'loose',
-		) || '';
-		return ($self->has_older_version_than ($pkg, $vers,  $filter, \@dist_version ));
+		return ($self->has_older_version_than ($pkg, $vers, \@dist_version ));
 	}
 	else {
 		return (1) ;
@@ -677,20 +705,13 @@ sub check_versioned_dep {
 }
 
 sub has_older_version_than {
-    my ($self, $pkg, $vers, $filter, $dist_version ) = @_;
-
-    $logger->debug("using filter $filter") if $filter;
-    my $regexp = $deb_release_h{$filter} ;
-
-    $logger->debug("using regexp $regexp") if defined $regexp;
+    my ($self, $pkg, $vers, $dist_version ) = @_;
 
     my @list ;
     my $has_older = 0;
     while (@$dist_version) {
         my ($d,$v) = splice @$dist_version,0,2 ;
 
-        next if defined $regexp and $d =~ $regexp ;
-
         push @list, "$d -> $v;" ;
 
         if ($vs->compare($vers,$v) > 0 ) {
@@ -701,14 +722,14 @@ sub has_older_version_than {
     $logger->debug("$pkg $vers has_older is $has_older (@list)");
 
     return 1 if $has_older ;
-    return wantarray ? (0,@list) : 0 ;
+    return (0,@list) ;
 }
 
 #
 # New subroutine "check_essential_package" extracted - Thu Aug 30 14:14:32 2012.
 #
 sub check_or_fix_essential_package {
-    my ( $self, $apply_fix, $dep_info ) = @_;
+    my ( $self, $apply_fix, $dep_info, $msgs ) = @_;
     my $pkg = $dep_info->{name};
     my ($oper, $vers) = @{ $dep_info->{dep} || []};
     $logger->debug("called with '", scalar $self->struct_to_dep($dep_info), "' and fix $apply_fix") if $logger->is_debug;
@@ -722,12 +743,13 @@ sub check_or_fix_essential_package {
 
     if ($is_essential and not defined $oper) {
         $logger->debug( "found unversioned dependency on essential package: $pkg");
+        my $msg = "unnecessary unversioned dependency on essential package: $pkg";
         if ($apply_fix) {
             %$dep_info = ();
             $logger->info("fix: removed unversioned essential dependency on $pkg");
+            push $msgs->@*, $msg;
         }
         else {
-            my $msg = "unnecessary unversioned dependency on essential package: $pkg";
             $self->add_warning($msg);
             $self->{nb_of_fixes}++;
             $logger->info("will warn: $msg (fix++)");
@@ -741,7 +763,7 @@ my %pkg_replace = (
 ) ;
 
 sub check_or_fix_pkg_name {
-    my ( $self, $apply_fix, $dep_info, $old ) = @_;
+    my ( $self, $apply_fix, $dep_info, $old, $msgs ) = @_;
     my $pkg = $dep_info->{name};
 
     $logger->debug("called with '", scalar $self->struct_to_dep($dep_info), "' and fix $apply_fix")
@@ -749,12 +771,13 @@ sub check_or_fix_pkg_name {
 
     my $new = $pkg_replace{$pkg} ;
     if ( $new ) {
+        my $msg = "dubious package name: $pkg. Preferred package is $new";
         if ($apply_fix) {
             $logger->info("fix: changed package name from $pkg to $new");
             $dep_info->[0] = $pkg = $new;
+            push $msgs->@*, $msg;
         }
         else {
-            my $msg = "dubious package name: $pkg. Preferred package is $new";
             $self-> add_warning ($msg);
             $self->{nb_of_fixes}++;
             $logger->info("will warn: $msg (fix++)");
@@ -778,55 +801,81 @@ sub check_or_fix_pkg_name {
 }
 
 sub check_or_fix_dep {
-    my ( $self, $apply_fix, $dep_info, $old ) = @_;
+    my ( $self, $apply_fix, $dep_info, $old, $msgs ) = @_;
     my $pkg = $dep_info->{name};
 
     $logger->debug("called with '", scalar $self->struct_to_dep($dep_info), "' and fix $apply_fix")
         if $logger->is_debug;
 
-    if(not defined $pkg) {
+    if (not defined $pkg) {
         # pkg may be cleaned up during fix
+        return;
     }
-    elsif ( $pkg eq 'debhelper' ) {
-        $self->check_debhelper_version( $apply_fix, $dep_info );
+
+    if ( $pkg eq 'debhelper' ) {
+        return $self->check_debhelper_version( $apply_fix, $dep_info, $msgs );
+    }
+
+    my ( $vers_dep_ok, @list ) =  $self->check_versioned_dep( $dep_info );
+    return if $vers_dep_ok;
+
+    if ($dep_info->{dep}[0] =~ /</) {
+        $self->warn_or_remove_dep ($apply_fix, $dep_info, \@list, $msgs) ;
     }
     else {
-		my ( $vers_dep_ok, @list ) =  $self->check_versioned_dep( $dep_info );
-		$self->warn_or_remove_vers_dep ($apply_fix, $dep_info, \@list) unless $vers_dep_ok ;
+        $self->warn_or_remove_vers_dep ($apply_fix, $dep_info, \@list, $msgs) ;
     }
 }
 
 sub warn_or_remove_vers_dep {
-    my ( $self, $apply_fix, $dep_info, $list ) = @_;
+    my ( $self, $apply_fix, $dep_info, $list, $msgs ) = @_;
     my $pkg = $dep_info->{name};
-    my ($oper, $vers) = @{ $dep_info->{dep} || []};
 
+    my $msg = "unnecessary greater-than versioned dependency: ". $self-> struct_to_dep($dep_info)
+        . ". Debian has @$list";
     if ($apply_fix) {
-        delete $dep_info->{dep};    # remove versioned dep, notify_change called in check_value
-        $logger->info("fix: removed versioned dependency from $dep_info->{name} -> $pkg");
+        delete $dep_info->{dep};    # remove greater-than versioned dep, notify_change called in check_value
+        $logger->info("fix: removed greater-than versioned dependency from $pkg");
+        push $msgs->@*, $msg;
     }
     else {
         $self->{nb_of_fixes}++;
-        my $msg = "unnecessary versioned dependency: ". $self-> struct_to_dep($dep_info)
-            . ". Debian has @$list";
         $self->add_warning( $msg);
         $logger->info("will warn: $msg (fix++)");
     }
 }
 
-use vars qw/%cache/ ;
+sub warn_or_remove_dep {
+    my ( $self, $apply_fix, $dep_info, $list, $msgs ) = @_;
+    my $pkg = $dep_info->{name};
+
+    my $msg = "unnecessary older-than versioned dependency: ". $self-> struct_to_dep($dep_info)
+        . ". Debian has @$list";
+    if ($apply_fix) {
+        %$dep_info = ();    # remove whole dependency
+        $logger->info("fix: removed dependency $pkg");
+        push $msgs->@*, $msg;
+    }
+    else {
+        $self->{nb_of_fixes}++;
+        $self->add_warning( $msg);
+        $logger->info("will warn: $msg (fix++)");
+    }
+}
+
+use vars qw/%cache $use_test_cache/ ;
 
 # Set up persistence
 my $cache_file_name = $ENV{HOME}.'/.config_model_depend_cache' ;
 
 # this condition is used during tests
-if (not %cache) {
+if (not $use_test_cache) {
     tie %cache => 'DB_File', $cache_file_name,
 }
 
 # required to write data back to DB_File
 END {
-    untie %cache ;
+    untie %cache unless $use_test_cache;
 }
 
 my $cache_expire_date = time - 24 * 60 * 60 * 7 ;
@@ -999,10 +1048,9 @@ syntax as described in http://www.debian.org/doc/debian-policy/ch-relationships.
 =item *
 
 Whether the version specified with C<< > >> or C<< >= >> is necessary.
-This module will check with Debian server whether older versions can be
+This module checks with Debian server whether older versions can be
 found in Debian old-stable or not. If no older version can be found, a
-warning will be issued. Note a warning will also be sent if the package
-is not found on madison and if the package is not virtual.
+warning is issued (unless the package is known to be virtual)
 
 =item *
 
