@@ -14,6 +14,13 @@
         'description' => 'This element contains a set of parameters to tune the behavior of this dpkg editor. You can for instance specify e-mail replacements. These parameters are stored in ~/.dpkg-meta.yml or ~/.local/share/.dpkg-meta.yml. These parameters can be applied to all Debian packages you maintain in this unix account.',
         'type' => 'node'
       },
+      'compat',
+      {
+        'default' => '9',
+        'description' => 'compat file defines the debhelper compatibility level',
+        'type' => 'leaf',
+        'value_type' => 'integer'
+      },
       'control',
       {
         'config_class_name' => 'Dpkg::Control',
@@ -22,11 +29,54 @@
       },
       'rules',
       {
+        'default' => '#!/usr/bin/make -f
+# See debhelper(7) (uncomment to enable)
+# output every command that modifies files on the build system.
+#DH_VERBOSE = 1
+
+# see EXAMPLES in dpkg-buildflags(1) and read /usr/share/dpkg/*
+DPKG_EXPORT_BUILDFLAGS = 1
+include /usr/share/dpkg/default.mk
+
+# see FEATURE AREAS in dpkg-buildflags(1)
+#export DEB_BUILD_MAINT_OPTIONS = hardening=+all
+
+# see ENVIRONMENT in dpkg-buildflags(1)
+# package maintainers to append CFLAGS
+#export DEB_CFLAGS_MAINT_APPEND  = -Wall -pedantic
+# package maintainers to append LDFLAGS
+#export DEB_LDFLAGS_MAINT_APPEND = -Wl,--as-needed
+
+# main packaging script based on dh7 syntax
+%:
+	dh $@',
         'description' => 'debian/rules is a makefile containing all instructions required to build a debian package.',
         'summary' => 'package build rules',
         'type' => 'leaf',
         'value_type' => 'string'
       },
+      'changelog',
+      {
+        'compute' => {
+          'allow_override' => '1',
+          'formula' => '$pkg_name (0.001-1) UNRELEASED; urgency=medium
+
+  * ...
+
+ -- $name <$mail>  Wed, 18 Jan 2017 18:28:23 +0100
+',
+          'variables' => {
+            'mail' => '! my_config email',
+            'name' => '! my_config fullname',
+            'pkg_name' => '! control source Source'
+          }
+        },
+        'description' => 'Dummy changelog entry with a dummy date.
+Don\'t forget to change the version
+number. Use L<dch> command to update.',
+        'type' => 'leaf',
+        'value_type' => 'string'
+      },
       'scan-copyright-patterns',
       {
         'config_class_name' => 'Dpkg::Copyright::ScanPatterns',
@@ -63,6 +113,29 @@ details',
         'summary' => 'copyright and license information',
         'type' => 'node'
       },
+      'install',
+      {
+        'cargo' => {
+          'config_class_name' => 'Dpkg::Install',
+          'type' => 'node'
+        },
+        'description' => 'List the files to install into each package and the directory they should be installed to.',
+        'index_type' => 'string',
+        'migrate_keys_from' => '- control binary',
+        'type' => 'hash'
+      },
+      'not-installed',
+      {
+        'cargo' => {
+          'type' => 'leaf',
+          'value_type' => 'uniline'
+        },
+        'description' => 'List the files that are deliberately not installed in any binary package. Paths listed in this file are (only) ignored by the check done via --list-missing (or --fail-missing). However, it is not a method to exclude files from being installed. Please use --exclude for that.
+
+Please keep in mind that dh_install will not expand wildcards in this file.
+',
+        'type' => 'list'
+      },
       'source',
       {
         'config_class_name' => 'Dpkg::Source',
@@ -85,6 +158,26 @@ details',
         'summary' => 'list of files to clean',
         'type' => 'list'
       },
+      'bugfiles',
+      {
+        'cargo' => {
+          'config_class_name' => 'Dpkg::BugFiles',
+          'type' => 'node'
+        },
+        'follow_keys_from' => '- control binary',
+        'index_type' => 'string',
+        'type' => 'hash'
+      },
+      'package-scripts',
+      {
+        'cargo' => {
+          'config_class_name' => 'Dpkg::PackageScripts',
+          'type' => 'node'
+        },
+        'follow_keys_from' => '- control binary',
+        'index_type' => 'string',
+        'type' => 'hash'
+      },
       'patches',
       {
         'cargo' => {
@@ -97,13 +190,6 @@ details',
         'summary' => 'Debian patches applied to original sources',
         'type' => 'hash'
       },
-      'compat',
-      {
-        'default' => '9',
-        'description' => 'compat file defines the debhelper compatibility level',
-        'type' => 'leaf',
-        'value_type' => 'integer'
-      },
       'dirs',
       {
         'cargo' => {
@@ -129,6 +215,12 @@ It is best to try to run the installation first and only use this if you run int
 
 By default, it will include all existing files in the top-level source directory that are called BUGS, README*, TODO etc. ',
         'type' => 'list'
+      },
+      'watch',
+      {
+        'description' => 'watch file used by L<uscan> to monitor upstream sources',
+        'type' => 'leaf',
+        'value_type' => 'string'
       }
     ],
     'license' => 'LGPL2',
@@ -138,8 +230,7 @@ By default, it will include all existing files in the top-level source directory
         'auto_create' => '1',
         'auto_delete' => '1',
         'backend' => 'Dpkg',
-        'config_dir' => 'debian',
-        'file' => 'clean'
+        'config_dir' => 'debian'
       }
     ]
   }
