@@ -22,6 +22,12 @@ For details, see L<section 5.7 of Debian policy|https://www.debian.org/doc/debia
     'element' => [
       'Source',
       {
+        'compute' => {
+          'allow_override' => '1',
+          'formula' => '$ENV{PWD} =~ m!/([^/]+)$!; $1;',
+          'use_eval' => '1'
+        },
+        'description' => 'Source package name. Defaults to the name of the current directory.',
         'mandatory' => '1',
         'match' => '\\w[\\w+\\-\\.]{1,}',
         'summary' => 'source package name',
@@ -183,7 +189,7 @@ in Debian developer reference.',
           'optional',
           'extra'
         ],
-        'default' => 'extra',
+        'default' => 'optional',
         'help' => {
           'extra' => 'This contains all packages that conflict with others with required, important, standard or optional priorities, or are only likely to be useful if you already know what they are or have specialized requirements (such as packages containing only detached debugging symbols).',
           'important' => 'Important programs, including those which one would expect to find on any Unix-like system. If the expectation is that an experienced Unix person who found it missing would say "What on earth is going on, where is foo?", it must be an important package.[5] Other packages without which the system will not run well or be usable must also have priority important. This does not include Emacs, the X Window System, TeX or any other large applications. The important packages are just a bare minimum of commonly-expected and necessary tools.',
@@ -223,11 +229,29 @@ to build architecture-dependent binaries ("Architecture: any" or specific archit
 
 Technically, these packages must be installed for the following build targets: clean, build-arch, and binary-arch. See L<build target|https://www.debian.org/doc/debian-policy/ch-source.html#s-debianrules> doc and L<details on this field|https://www.debian.org/doc/debian-policy/ch-relationships.html#s-sourcebinarydeps>.
 
-On the other hand, the list of packages that must be installed to build architecture-independent binaries ("Architecture: all") should be listed in "Build-Depends-Indep" field.',
+On the other hand, the list of packages that must be installed to build architecture-independent binaries ("Architecture: all") should be listed in "Build-Depends-Indep" field.
+
+Including a dependency in this field does not have the exact same effect as including it in both Build-Depends-Arch and Build-Depends-Indep, because the dependency also needs to be satisfied when building the source package.
+
+See also L<deb-src-control|https://manpages.debian.org/unstable/dpkg-dev/deb-src-control.5.en.html> man page',
         'duplicates' => 'warn',
         'summary' => 'List of package required during clean or build of architecture-dependent packages',
         'type' => 'list'
       },
+      'Build-Depends-Arch',
+      {
+        'cargo' => {
+          'class' => 'Config::Model::Dpkg::Dependency',
+          'type' => 'leaf',
+          'value_type' => 'uniline'
+        },
+        'description' => 'Same as Build-Depends, but these pakcages are only needed when building the architecture dependent packages. The Build-Depends are also installed in this case. 
+
+See L<deb-src-control man page|https://manpages.debian.org/unstable/dpkg-dev/deb-src-control.5.en.html> for details',
+        'duplicates' => 'warn',
+        'summary' => 'List of package required to build architecture-dependent packages',
+        'type' => 'list'
+      },
       'Build-Depends-Indep',
       {
         'cargo' => {
@@ -266,11 +290,28 @@ to build architecture-dependent binaries ("Architecture: any" or specific archit
 
 Technically, these packages must B<not> be installed for the following build targets: clean, build-arch, and binary-arch. See L<build target|https://www.debian.org/doc/debian-policy/ch-source.html#s-debianrules> doc and L<details on this field|https://www.debian.org/doc/debian-policy/ch-relationships.html#s-sourcebinarydeps>.
 
-On the other hand, the list of packages that must B<not> be installed to build architecture-independent binaries ("Architecture: all") should be listed in "Build-Conflicts-Indep" field.',
+On the other hand, the list of packages that must B<not> be installed to build architecture-independent binaries ("Architecture: all") should be listed in "Build-Conflicts-Indep" field.
+
+Including a dependency in this list has the same effect as including it in both Build-Conflicts-Arch and Build-Conflicts-Indep, with the additional effect of being used for source-only builds. 
+
+See L<deb-src-control man page|https://manpages.debian.org/unstable/dpkg-dev/deb-src-control.5.en.html> for details.',
         'duplicates' => 'warn',
         'summary' => 'List of package not wanted during clean or build of architecture-dependent packages',
         'type' => 'list'
       },
+      'Build-Conflicts-Arch',
+      {
+        'cargo' => {
+          'type' => 'leaf',
+          'value_type' => 'uniline'
+        },
+        'description' => 'List of packages that must be missing (i.e. B<not> installed) to build archictecture dependent binaries 
+
+See L<deb-src-control man page|https://manpages.debian.org/unstable/dpkg-dev/deb-src-control.5.en.html> for details.',
+        'duplicates' => 'warn',
+        'summary' => 'List of package not wanted during build of architecture dependent packages',
+        'type' => 'list'
+      },
       'Build-Conflicts-Indep',
       {
         'cargo' => {
@@ -286,9 +327,32 @@ Note that packages not wanted during "clean" phase must be declared in "Build-Co
         'summary' => 'List of package not wanted during build of architecture-independent packages',
         'type' => 'list'
       },
+      'Built-Using',
+      {
+        'cargo' => {
+          'class' => 'Config::Model::Dpkg::Dependency',
+          'type' => 'leaf',
+          'value_type' => 'uniline'
+        },
+        'description' => ' Some binary packages incorporate parts of other packages when built but do not have to depend on those packages. Examples include linking with static libraries or incorporating source code from another package during the build. In this case, the source packages of those other packages are a required part of the complete source (the binary package is not reproducible without them).
+
+A Built-Using field must list the corresponding source package for any such binary package incorporated during the build, including an B<exactly equal> ("=") version relation on the version that was used to build that binary package[57].
+
+A package using the source code from the gcc-4.6-source binary package built from the gcc-4.6 source package would have this field in its control file:
+
+     Built-Using: gcc-4.6 (= 4.6.0-11)
+
+A package including binaries from grub2 and loadlin would have this field in its control file:
+
+     Built-Using: grub2 (= 1.99-9), loadlin (= 1.6e-1)
+',
+        'duplicates' => 'warn',
+        'summary' => 'Additional source packages used to build the binary',
+        'type' => 'list'
+      },
       'Standards-Version',
       {
-        'default' => '3.9.8',
+        'default' => '4.1.0',
         'description' => 'This field indicates the Debian policy version number this package complies to.
 
 Before updating this field, please read L<upgrading-checklist|https://www.debian.org/doc/debian-policy/upgrading-checklist.html> to know what changes came with a new policy version number and apply the required changes (if any) to your package.',
