@@ -92,7 +92,7 @@ sub _warn ($msg) {
 
 my %default ;
 # from licensecheck.pl
-$default{ignore}= qr!
+$default{ignore}= << 'EOR';
 # Ignore general backup files
 ~$|
 # Ignore emacs recovery files
@@ -105,13 +105,16 @@ $default{ignore}= qr!
 (?:^|/)(?:DEADJOE|\.cvsignore|\.arch-inventory|\.bzrignore|\.gitignore)$|
 # File or directory names that should be ignored
 (?:^|/)(?:CVS|RCS|\.pc|\.deps|\{arch\}|\.arch-ids|\.svn|\.hg|_darcs|\.git|
-\.shelf|_MTN|\.bzr(?:\.backup|tags)?)(?:$|/.*$)
-!x;
+\.shelf|_MTN|\.bzr(?:\.backup|tags)?)(?:$|/.*$)|
+# skip debian files that are too confusing or too short
+(?:^|/)debian/((fix.scanned.)?copyright|changelog|NEWS|compat|.*yml|docs|source|patches/series)
+EOR
 
 # also from licensecheck
-$default{check} =
-    qr!
-    \.(                          # search for file suffix
+$default{check} = << 'EOR2' ;
+    (^|/)\w+$                 # scripts, AUTHORS files
+    |^README                   # README.* files
+    |\.(                          # search for file suffix
         c(c|pp|xx)?              # c and c++
        |h(h|pp|xx)?              # header files for c and c++
        |S
@@ -119,6 +122,9 @@ $default{check} =
        |f(77|90)?
        |go
        |groovy
+       |in|am                    # file to pre-process
+       |m4
+       |\d                       # man pages
        |lisp
        |scala
        |clj
@@ -144,10 +150,10 @@ $default{check} =
        |(c|l)?hs
      )
     $
-   !x;
+EOR2
 
 # cleanup the regexp
-map { s/#.*\n//g;  s/\n//g; } values %default;
+map { s/#.*\n//g;  s/[\s\n]+//g; } values %default;
 
 # option to skip UNKNOWN ?
 # load a file to override some entries ?
@@ -157,6 +163,7 @@ sub scan_files ( %args ) {
 
     my $current_dir = $args{from_dir} || path('.');
 
+    my %regexps = %default; # default is needed during tests 
     my @lines ;
     if (my $file = $ENV{COPYRIGHT_SCANNER_INPUT}) {
         @lines = path($file)->lines_utf8 ; # for tests
@@ -165,32 +172,36 @@ sub scan_files ( %args ) {
         @lines = $args{in}->lines_utf8; # for other tests
     }
     else {
-        my @runs = (['--skipped'], [qw!-c (?i:readme|license|copying).*!]);
-
+        my $scan_data = {};
         my $debian = $current_dir->child('debian');
         my $scan_patterns = $debian->child("copyright-scan-patterns.yml");
 
         if ($debian->is_dir and $scan_patterns->is_file) {
             my $yaml = $scan_patterns->slurp_utf8;
             # { check => { suffixes => [ js pl ], pattern => '[A-Z]*'}, ignore => { suffixes => [ jpg png ], pattern => 'foo.*'} }
-            my $scan_data = Load $yaml;
-            my @opts = ();
-            foreach my $what (qw/check ignore/) {
-                my $data = $scan_data->{$what} or next;
-                my $reg = join( '|' , (map { '\.'.$_.'$'} @{$data->{suffixes} || []}), @{ $data->{pattern} || []}, $default{$what});
-                push @opts, '--skipped', "--$what=$reg";
-            }
-            @runs = \@opts ;
+            $scan_data = Load $yaml;
         }
 
-        foreach my $opts ( @runs ) {
-            my $pipe = IO::Pipe->new();
-            my @cmd = (qw/licensecheck --encoding utf8 --copyright --machine --recursive/,  @$opts, '.');
-            $pipe->reader(@cmd);
-            binmode($pipe, ":encoding(UTF-8)");
-            push @lines, $pipe->getlines;
-            $pipe->close;
+        # licensecheck --check is broken ( #842368 ), so --skipped option is useless.
+        # let's scan everything and skip later
+        foreach my $what (qw/check ignore/) {
+            my $data = $scan_data->{$what} or next;
+            my $reg = join(
+                '|' ,
+                (map { '\.'.$_.'$'} @{$data->{suffixes} || []}),
+                @{ $data->{pattern} || []},
+                $default{$what}
+            );
+            $regexps{$what} = qr/$reg/;
         }
+
+        my $pipe = IO::Pipe->new();
+        my @cmd = (qw/licensecheck --encoding utf8 --copyright --machine --recursive/,
+                   '--ignore='.$regexps{ignore}, '.');
+        $pipe->reader(@cmd);
+        binmode($pipe, ":encoding(UTF-8)");
+        @lines = $pipe->getlines;
+        $pipe->close;
     }
 
     my $fill_blank_data = __load_fill_blank_data($current_dir);
@@ -199,15 +210,22 @@ sub scan_files ( %args ) {
     my $files = {};
     my $id = 1;
     my @no_info_list;
+    my @skipped;
 
     foreach my $line (sort @lines) {
         chomp $line;
         # say "found: $line";
-        my ($f,$l,$c) = split /\t/, $line; 
+        my ($f,$l,$c) = split /\t/, $line;
         $f =~ s!\./!!;
-        next if $f =~ m!debian/(copyright|changelog)$!; # too confusing
         next if $files->{$f}; # file already parsed
 
+        # filter out files user does not want checked
+        if ( $f !~ $regexps{check} ) {
+            # the ignore test should not be usefull as the files are ignored by licensecheck
+            push @skipped, $f unless $f =~ $regexps{ignore};
+            next;
+        }
+
         $l =~ s/([*?\\])/\\$1/g;
         $l =~ s/\s*\(unversioned\/unknown version\)//g;
         $l =~ s/\s*\(with incorrect FSF address\)//g;
@@ -221,11 +239,15 @@ sub scan_files ( %args ) {
         $l =~ s/\bBSD \((\d) clause\)/BSD-$1-clause/g;
         $l =~ s/\bpublic domain\b/public-domain/ig;
 
+
         # deduplicate license keywords and add 'or' keyword where missing
         # this is fragile. may need to change license-check to output license keyword
         my %tmp_l = map { ( $_ => 1) ;} grep { $_ ne 'or' } split(/ /,$l);
         $l = join(' or ', sort keys %tmp_l) ;
 
+        # Handle license like "under the same term as Perl"
+        $l =~ s/perl/Artistic or GPL-1+/i;
+
         $c =~ s/'//g;
         $c =~ s/^&copy;\s*//;
         $c =~ s/(?<=\b\d{4})\s*-\s*\d{4}(?=\s*-\s*(\d{4})\b)//g;
@@ -277,6 +299,16 @@ sub scan_files ( %args ) {
         # say "Storing '$f' : '$c' '$l' has_info: $has_info id ".$files->{$f};
     }
 
+    if (@skipped) {
+        my $msg= "The following files were skipped:\n";
+        map {
+             $msg .= "- $_\n";
+         } @skipped;
+        $msg .= "You may want to add a line in debian/copyright-scan-patterns.yml\n"
+            ."or ask the author to add more default patterns to scan\n\n";
+        _warn $msg;
+    }
+
     if (@no_info_list) {
         my $msg= "The following paths are missing information:\n";
         map {my ($p,@i) = $_->@*;
@@ -286,7 +318,7 @@ sub scan_files ( %args ) {
         _warn $msg;
     }
 
-                my @notused = grep { ! $fill_blank_data->{$_}{used} and $_; } sort keys %$fill_blank_data ;
+    my @notused = grep { ! $fill_blank_data->{$_}{used} and $_; } sort keys %$fill_blank_data ;
     if (@notused) {
         _warn "Warning: the following entries from fill.copyright.blanks.yml were not used\n- '"
             .join("'\n- '",@notused)."'\n";
@@ -312,7 +344,7 @@ sub scan_files ( %args ) {
 }
 
 sub __split_copyright ($c) {
-    my ($years,$owner) = $c =~ /([\s,\d-]+)(.*)/;
+    my ($years,$owner) = $c =~ /^(\d\d[\s,\d-]+)(.*)/;
     # say "undef year in $c" unless defined $years;
     return unless defined $years;
     my @data = split /(?<=\d)[,\s]+/, $years;
@@ -430,7 +462,7 @@ sub __squash_copyrights_years ($copyrights_by_id) {
 
         my ($l,@owners) = split /\|/, $owner_license;
 
-        # create new copyright info with coaslesced years
+        # create new copyright info with coalesced years
         my @squashed_c = __coalesce_copyright_years(\@entries,\@owners) ;
         next unless @squashed_c ; # give up this entry when problem
 
@@ -468,7 +500,7 @@ sub __coalesce_copyright_years($entries, $owners) {
         my ($id, @years) = $entry->@* ;
 
         for (my $i = 0; $i < @years; $i++) {
-            return () if $years[$i] =~ /[^\d,\s-]/;
+            return () unless $years[$i] =~ /^[0-9,\s-]+$/; # detect empty year range
             my $span = $ranges_of_years[$i] //= Array::IntSpan->new();
             return () unless $span; # bail out in case of problems
             eval {
@@ -636,12 +668,14 @@ the list of files.
 
 =head1 Selecting or ignoring files to scan
 
-By default, the decision whether to scan a file or not is left to L<licensecheck>.
+By default, scanner scans source files with known suffixes (like .c
+.pl ...), README, scripts and skip backup files.
 
 You can change this behavior in file
-C<debian/copyright-scan-patterns.yml>. This YAML file can
-contain a list of suffixes or patterns to scan or to ignore that are added to licensecheck's list. Any file
-that is not scanned or ignored will be shown as "skipped".
+C<debian/copyright-scan-patterns.yml>. This YAML file can contain a
+list of suffixes or patterns to scan or to ignore that are added to
+the default list. Any file that is not scanned or ignored will be
+shown as "skipped".
 
 The file must have the following structure (all fields are optional
 and order does not matter):
@@ -664,6 +698,8 @@ and order does not matter):
 
 Do not specify the dot with the suffixes. This will be added by the scanner.
 
+Note that a file that match both "check" and "ignore" pattern is ignored.
+
 =head1 Filling the blanks
 
 Sometimes, upstream coders are not perfect: some source files cannot
