@@ -20,10 +20,12 @@ providing the following file:
         check => {
             'source Source',          "libdist-zilla-plugins-cjm-perl",
             'source Build-Depends:0', "debhelper (>= 7)",
-            'source Build-Depends-Indep:0', "libcpan-meta-perl | perl (>= 5.13.10)",     # fixed
+            # not really happy about the versioned dep of libmeta-cpan-perl created ex-nihilo.
+            # this is a side effect of other compromises in dual dependency handling which should be harmless
+            'source Build-Depends-Indep:0', "libcpan-meta-perl (>= 2.110440) | perl (>= 5.13.10)",     # fixed
             'source Build-Depends-Indep:1', "libdist-zilla-perl",    # fixed
             'source Build-Depends-Indep:5', "libpath-class-perl",
-            'source Build-Depends-Indep:6', "libmodule-build-perl (>= 0.360000)", # fixed
+            'source Build-Depends-Indep:6', "libmodule-build-perl (>= 0.360000) | perl (<< 5.19.0)", # fixed
             'source Build-Depends-Indep:7', "udev [linux-any] | makedev [linux-any]",
             'source X-Comment' => qr/Debian #810023/,
             'binary:libdist-zilla-plugins-cjm-perl X-Comment' => qr/Debian #810023/,
@@ -103,9 +105,12 @@ providing the following file:
     {
         name => 'sdlperl',
         load => 'source Uploaders:2="Sam Hocevar (Debian packages) <sam@zoy.org>"',
-        load_warnings => [ ( qr/Warning/) x 11 ],
+        load_warnings => [ ( qr/Warning/) x 12 ],
         load_check => 'skip',
-        check => { 'binary:libsdl-perl Depends:2' => '${misc:Depends}' },
+        check => {
+            'binary:libsdl-perl Depends:2' => '${misc:Depends}',
+            'binary:libsdl-perl Conflicts:2' => undef,
+        },
         apply_fix => 1,
     },
     {
@@ -132,8 +137,8 @@ providing the following file:
     {
         # test for #682730
         name => 'libclass-meta-perl',
-        load_warnings => [ ( qr/Vcs-Git/) x 2 ],
-         check => { 'source Build-Depends-Indep:1' => 'libclass-isa-perl | perl (<< 5.10.1-13)' },
+        load_warnings => [ qr/dual life/,  ( qr/Vcs-Git/) x 2, qr/dual life/ ],
+         check => { 'source Build-Depends-Indep:1' => 'libclass-isa-perl | perl (<< 5.11.0)' },
         apply_fix => 1,
     },
     {
@@ -205,6 +210,8 @@ providing the following file:
 
 my $cache_file = 't/model_tests.d/dependency-cache.txt';
 
+$Config::Model::Dpkg::Dependency::use_test_cache = 1;
+
 my $ch = new IO::File "$cache_file";
 foreach ($ch->getlines) {
     chomp;
